#include "DS18B20.h"

// /**********************************************************
// *DS18B20 初始化
// *函数名称:DS18B20_WriteData()
// *说明：本初始化程序可以不要，因为18B20 在出厂时就被配置为12 位精度了
// **********************************************************/

void DS18B20_Init() {
	DS18B20_Reset();
	DS18B20_WriteData(0xCC); // 跳过ROM
	DS18B20_WriteData(0x4E); // 写暂存器
	DS18B20_WriteData(0x20); // 往暂存器的第三字节中写上限值
	DS18B20_WriteData(0x00); // 往暂存器的第四字节中写下限值
	DS18B20_WriteData(0x7F); // 将配置寄存器配置为12 位精度
	DS18B20_Reset();
}


/**********************************************************
*DS18B20 复位及存在检测(通过存在脉冲可以判断DS18B20 是否损坏)
*函数名称:DS18B20_Reset()
*说明:函数返回一个位标量(0 或1)flag=0 存在,反之flag=1 不存在
**********************************************************/
bit DS18B20_Reset() {
	U8 i;
	bit flag;
	DS18B20_DQ = 0; //拉低总线
	for (i = 240; i > 0; i--); //延时480 微秒,产生复位脉冲
	DS18B20_DQ = 1; //释放总线
	for (i = 40; i > 0; i--); //延时80 微秒对总线采样
	flag = DS18B20_DQ; //对数据脚采样
	for (i = 200; i > 0; i--); //延时400 微秒等待总线恢复
	return flag; //根据flag 的值可知DS1820 是否存在或损坏 ，可加声音告警提示DS1820 故障
}

/**********************************************************
*写数据到DS18B20
*函数名称:DS18B20_WriteData()
**********************************************************/
void DS18B20_WriteData(U8 wData) {
    U8 i, j;
    for (i = 8; i > 0; i--) {
        DS18B20_DQ = 0; //拉低总线,产生写信号
        for (j = 2; j > 0; j--); //延时4us
        DS18B20_DQ = wData & 0x01; //发送1 位
        for (j = 30; j > 0; j--); //延时60us,写时序至少要60us
        DS18B20_DQ = 1; //释放总线,等待总线恢复
        wData >>= 1; //准备下一位数据的传送
    }
}

/**********************************************************
*从DS18B20 中读出数据
*函数名称:DS18B20_ReadData()
**********************************************************/
U8 DS18B20_ReadData() {
    U8 i, j, TempData;
    for (i = 8; i > 0; i--) {
        TempData >>= 1;
        DS18B20_DQ = 0; //拉低总线,产生读信号
        for (j = 2; j > 0; j--); //延时4us
        DS18B20_DQ = 1; //释放总线,准备读数据
        for (j = 4; j > 0; j--); //延时8 微秒读数据
        if (DS18B20_DQ == 1) { 
            TempData |= 0x80;
        }
        for (j = 30; j > 0; j--); //延时60us
        DS18B20_DQ = 1; //拉高总线,准备下一位数据的读取.
    }
    return TempData;//返回读到的数据
}